/*****************************************************************************\
*
* elektron.h
*
* LINAC - simulation of using LINAC for acceleration of electron
*
* Authors: Ondrej Peterka (xpeter08 _AT stud.fit.vutbr.cz)
*          PCJohn (peciva _AT fit.vutbr.cz)
* Contributors:
*
* ----------------------------------------------------------------------------
*
* THIS SOFTWARE IS NOT COPYRIGHTED
*
* This source code is offered for use in the public domain.
* You may use, modify or distribute it freely.
*
* This source code is distributed in the hope that it will be useful but
* WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
* DISCLAIMED.  This includes but is not limited to warranties of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
*
* If you find the source code useful, authors will kindly welcome
* if you give them credit and keep their names with their source code,
* but do not feel to be forced to do so.
*
\*****************************************************************************/
#include "elektron.h"
#include "scena.h"

#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoGroup.h>
#include <Inventor/nodes/SoTranslation.h>
#include <Inventor/nodes/SoTexture2.h>
#include <Inventor/nodes/SoMaterial.h>
#include <Inventor/nodes/SoSphere.h>
#include <Inventor/actions/SoSearchAction.h> 
#include <Inventor/sensors/SoTimerSensor.h> 
#include <Inventor/nodes/SoText2.h>

#include <iostream>


#include <vector>
#include <list>
using namespace std;
extern float RightSizeOfRoura;
extern std::vector<float> VecBoost; //vector meznich hodnot jednotlivych elektrod
extern bool switchColor; //na prepinani barev na elektrodach

const double QEL = 1.602e-19;  // naboj castice
const double MEL = 9.109e-31; //hmotnost elektronu

extern double frekvence; //globalni
extern float MAXVolts;  //globalni
float velSim=1e5; //nastaveni meritka simulace->zpomaleni zrychleni animace

SbTime VoltLastTime=SbTime::zero(); //cas pro napeti pro ulozeni posledniho
SbTime pocDT; //uklada si prvni casovy posun
bool poprve=true; //spusteni napeti na prvni elektron
float VoltTime = 0; //cas pro napeti

SoTimerSensor * tSensor=NULL; //sensor pro zmenu barev elektrod

extern Elektron * pickedElectron; //vybrany elektron
extern SoText2 * enOut; //vypis energie
extern SoText2 * velOut; //vypis rychlosti
extern bool bReset; //byla vybrana volba pro reset?
extern bool bRedraw; //byla vybrana volba pro novou tvorbu LINACu


//posledni sepnuti timesensoru
SbTime lastTSensor=SbTime::zero();

std::list<Elektron*> vOut; //na odstraneni elektronu
std::list<Micro*> vOut_micros; //na mazani micros

extern std::list<SoMaterial *> VecElMat; //vector materilau pro zmeneni (elektrody a vedeni)

//////////callback funkce pro zmenu barev elektrod a vedeni
/////////////////////////////////////////////////////////////////
void cbTimerSensor(void * , SoSensor *sensor) {
	switchColor = !switchColor;

	//ulozim si cas sepnuti
	lastTSensor = SbTime::getTimeOfDay();

	bool lswitch = switchColor;



	for (list<SoMaterial*>::iterator ci=VecElMat.begin(); ci != VecElMat.end(); ++ci) {
		if (lswitch)
		  (*ci)->diffuseColor.setValue(SbColor(0.9f, 0.0f, 0.0f));
		else
		  (*ci)->diffuseColor.setValue(SbColor(.0f, 1.0f, .0f));

		lswitch = !lswitch;
	}



}

////////////konstruktor Micro
///////////////////////////////////////////////////////////
Micro::Micro(float pos_x, float pos_y, float pos_z,float size, SbColor color,
			    const char *textureName, 
			    float lVec, float vel_x, float vel_y, float vel_z,
				SoSeparator *sceneRoot) :
	Elektron(pos_x,pos_y,pos_z,size,color,textureName,lVec,vel_x,vel_y,vel_z,sceneRoot)
	{
		bPos_x =pos_x; //nastaveni pozice
	    bPos_y =pos_y;
		bPos_z =pos_z;

}

//////// konstruktor elektronu
/////////////////////////////////////////////////////////////////
Elektron::Elektron(float pos_x, float pos_y, float pos_z, float size, SbColor color,
			    const char *textureName, 
			    float lVec, float vel_x, float vel_y, float vel_z,
		     	SoSeparator * sceneRoot )
				: p_x(pos_x), p_y(pos_y), p_z(pos_z),
				  v_x(vel_x), v_y(vel_y), v_z(vel_z) 
{
	if (sceneRoot == NULL) { // jen pro jistotu		
		return;
	}

	SoSeparator *root = new SoSeparator; //root stromu elektronu

	translation = new SoTranslation; //posun
	translation->translation.setValue(p_x, p_y, p_z);
	root->addChild(translation);

	if (textureName != NULL) { //textura...nevyuziva se
		SoTexture2 *texture = new SoTexture2;
		texture->filename.setValue(textureName);
		root->addChild(texture);
	}

	material = new SoMaterial; //material
	material->diffuseColor.setValue(color);
	root->addChild(material);

	sphere = new SoSphere; //reprezentace elektronu
	sphere->radius.setValue(size);
	root->addChild(sphere);

	Croot=root; //ulozim si ukazatel na podstrom dfinujici dany elektron

	lastBoost=0; //kde se naposledy urychlil...nikdy :o)
	Energie=0;   //budeme prepokladta ze na zacatku ma nulovou energii
	velsimON=false; //zmenseni rychlosti na kontext simulace
	maxlv=lVec; //maximalni delka urazneeho vektoru (pro mikro)



	((SoGroup*)sceneRoot->getChild(0))->addChild(root);
}

///nastaveni pozice
//////////////////////////////////////////////////////////////////////
void Elektron::setPosition(float *array) {
	p_x = array[0];
	p_y = array[1];
	p_z = array[2];
	translation->translation.setValue(p_x, p_y, p_z);
}

///zjisteni pozice
/////////////////////////////////////////////////////////////////////
void Elektron::getPosition(float *array) {
	array[0] = p_x;
	array[1] = p_y;
	array[2] = p_z;
}

///funkce volana z callbacku Sooneshot sensoru-obstarava samotny pohyb elektornu
/////////////////////////////////////////////////////////////////////
void Elektron::updatePositions(list<Elektron*> * elektrons,
							   float sizeOfLINAC, const SbTime dt,
							   const SbTime lastTimeTick,SoSeparator * root) {

	float cur_pos[3], pos[3], a, a_x, a_y, a_z, dist, dist_x, dist_y, dist_z;
	float delta_time = dt.getValue() / velSim;



	list<Elektron*>::iterator citmp=NULL;



    int o=0;
	for (list<Elektron*>::iterator ci=elektrons->begin(); ci != elektrons->end(); ++ci) {
       o++;
    
	   //odstranim odkaz z minule iterace
	   if (citmp!=NULL) {
		   
		   elektrons->erase(citmp);
		   citmp=NULL;
	   }


	   //zjistim pozici
       (*ci)->getPosition(cur_pos);

	    //co jsou za desgtickou budou odtsraneny->dam ej do vectoru pro odtraneni
	   //rightsizeofroura+15 je poloha terciku
      if (cur_pos[2] > (RightSizeOfRoura+15) || cur_pos[2] <-30) {
	
			  vOut.push_back(*ci);

			  if (*ci == pickedElectron) pickedElectron= NULL;
			  citmp=ci;
			  continue;		  

	  }	  

  
	  float TmpVoltTime = VoltTime; //VoltTime pro vypocet

	  float moznapozice = cur_pos[2] + ((*ci)->v_z * delta_time);

	  //zkontroluju jestli jsem prosel elektrodou anebo jsem nakonci
	  if ((((*ci)->v_z>0) && moznapozice<VecBoost[(*ci)->lastBoost]) || 
		     (*ci)->lastBoost == (VecBoost.size()-1) ||
			 (((*ci)->v_z<0) && moznapozice>VecBoost[(*ci)->lastBoost])

			 ) { //neprosel
		  
		  // moznapozice bude nastavena jako dalsi pozice
		  cur_pos[2] = moznapozice;
		  (*ci)->setPosition(cur_pos); //nastavim novou pozici


	  } else { //prosel jsem elektrodou->vznika urychleni


          //vzdalenost elektronu od elektrody pred posunem
		  float s1 = VecBoost[(*ci)->lastBoost] - cur_pos[2];

		  // cas potrebny k tomu aby elektron dosahl elektrody starou rychlosti
		  float t1 = (s1 / (*ci)->v_z) ;

		  // vypocet Voltu na elektrode v case kdy elektron k elektrode dorazi
		     //jestlize poprve musi byt cas toven nule v okamziku kdy el. je v elektrode
		  if (poprve) {
			  VoltTime  = 0;
			  VoltTime -= t1; //aby byl nula pri prvnim boostu

			  TmpVoltTime = VoltTime;
			  poprve = false;
			  

			  //spusti se menneni barev na elektrodach
			  if (tSensor == NULL) {
			       tSensor = new SoTimerSensor(cbTimerSensor,NULL);
			  } 
			  switchColor=true;
			  tSensor->setBaseTime(SbTime::zero());
			  tSensor->setInterval(velSim/(2*frekvence));
			  if (tSensor->isScheduled()) tSensor->unschedule();
			  tSensor->schedule();
			  

		  } //end poprve

          //cas kdy elektron dorazi k elektrode
		  TmpVoltTime += t1;
		  
		  //vypocet napeti
          double Volts=(-MAXVolts)* cos(2*M_PI * frekvence *(TmpVoltTime));

	    	//simulace "stridaveho" proudu 
		  if (((*ci)->lastBoost%2)!=0) { //je lichy=>opacne napeti
				Volts=(-1)*Volts;
		  } 

		  //vypocet nove rychlosti
          (*ci)->Energie=((fabs((*ci)->Energie)/QEL)+((-1) *Volts))*QEL; //E2=(E1/Q+U2)Q
		  //v zavilsoti na energii bude dalsi pohyb
          (*ci)->v_z=((*ci)->Energie>0?sqrt(2*fabs((*ci)->Energie/MEL)):-sqrt(2*fabs((*ci)->Energie/MEL)));


		  //cas za ktery se dostane do dalsi pozice za elektrodou ...od elektrody pri nove rychlosti
		  float t2 = fabs(delta_time - t1);

		  // nova vzdalenost za elektrodou
		  float s2 = (*ci)->v_z * t2;

		  //posun a prirazeni rychlosti
		  cur_pos[2] += (s1 + s2);
		  (*ci)->setPosition(cur_pos); //nastavim novou pozici


		  //dopocitani TmpVoltTime ->pricte se tim padem cele dt
		  TmpVoltTime+=t2;

		  //zvyseni/snizeni lastboostu u daneho elektronu
		  ((*ci)->Energie>0?((*ci)->lastBoost)++:((*ci)->lastBoost)--);

		  ////////////////////////////////////////////////////////////
	/*	  		    std::cout<<"enregie0: "<<(*ci)->Energie<<"\n";
					std::cout<<"cislo: "<<o<<"\n";

					std::cout<<"v_z: "<<(*ci)->v_z<<"\n";
					std::cout<<"Volts0: "<<Volts<<"\n";
					std::cout<<"pozice0 Z: "<<(*ci)->p_z<<"\n";


		  		    std::cout<<"t1: "<<t1<<"\n";
					std::cout<<"t2: "<<t2<<"\n";
					std::cout<<"delta_time: "<<delta_time<<"\n";
					std::cout<<"s1: "<<s1<<"\n";
					std::cout<<"s2: "<<s2<<"\n\n";
					std::cout<<"NEXTboost: "<<(*ci)->lastBoost<<"\n\n";
	*/			
		  /////////////////////////////////////////////////////////////



	  } //end if - else

	  //vypis hodnot zvoleneho elektronu
	  if (enOut!=NULL && (pickedElectron !=NULL)) enOut->string = 0.5 * pow(pickedElectron->v_z,2) * MEL/QEL;//pickedElectron->Energie * (1/QEL);
	  if (velOut !=NULL && (pickedElectron !=NULL)) velOut->string = pickedElectron->v_z;
		 
	} //for

	VoltTime+=delta_time; //dokonceni casoveho prubehu

	if (citmp!=NULL) { // mazeme posledni zvoleny elektron pro vymazani ve vektoru 
		elektrons->erase(citmp);
	}


	deleteElektrons(root,elektrons);

} //end updatePositions


////zmena pozice micro castic
//////////////////////////////////////////////////////////////////
void Micro::updatePositionsMicro(std::list<Micro*> * micros,
						  const SbTime dt, const SbTime lastTimeTick,SoSeparator * root){

list<Micro*>::iterator citmp2=NULL;

//simulace pohybu, po urcite delce urazene drahy (random?) zanik
	for (list<Micro*>::iterator ci=micros->begin(); ci != micros->end(); ++ci) {

	   if (citmp2!=NULL) {
		  micros->erase(citmp2);
		   citmp2=NULL;
	   }

	  //co jsou za destickou budou odstraneny->dam ej do vectoru pro odstraneni
	   //rightsizeofroura+15 je poloha terciku
       if ((*ci)->getVecOver()) {

			  vOut_micros.push_back(*ci);
			  citmp2=ci;
			  continue;		  

	   }
		
		float cur_pos[3], npos[3];
	    float delta_time = dt.getValue() / velSim;

        (*ci)->getPosition(cur_pos);



		npos[0] = cur_pos[0] + ((*ci)->v_x * delta_time);
		npos[1] = cur_pos[1] + ((*ci)->v_y * delta_time);
		npos[2] = cur_pos[2] + ((*ci)->v_z * delta_time);

		(*ci)->setPosition(npos); //nastavim novou pozici


	}//end for

	if (citmp2!=NULL) {
		micros->erase(citmp2);
	}


	deleteMicros(root,micros);




}// end updatePositionsMicro





/////////zjisti otce zadaneho uzlu
//////////////////////////////////////////////////////////////////
SoGroup * getParent(SoNode * node, SoNode * root)
  {
    SoSearchAction sa;
    sa.setNode(node);
    sa.setInterest(SoSearchAction::FIRST); //cheme hned prvniho
    sa.apply(root); //jdeme od rootu
    SoPath * p = sa.getPath(); 
    assert(p && "not found");
    if (p->getLength() < 2) { return NULL; } // no parent
    return (SoGroup *)p->getNodeFromTail(1); //posledni z cesty
}






///smaze vsechny elektorny ve vektoru vOut a vyvola animaci burstu
//////////////////////////////////////////////////////////////////
void deleteElektrons(SoSeparator * r, list<Elektron*> * elektrons) {

	 for (list<Elektron*>::iterator ci=vOut.begin(); ci != vOut.end(); ++ci) {
	
		 //vytvoreni roztristeni elektronu na mensi castice
		 float cur_pos[3];
		 (*ci)->getPosition(cur_pos);
         if ((*ci)->v_z > 0 && !(bReset|| bRedraw)) //narazil zrejme do terciku 
		    createBurst(r,cur_pos[2],(*ci)->v_z); //animace

		 SoGroup * parent = getParent((*ci)->Croot,r); //otec
		 //zruseni elektronu
		 if (parent!=NULL) {
			 parent->removeChild((*ci)->Croot);	//odstraneni z grafu sceny	 
		 }


		
	 }

	 vOut.clear(); //bude prazdny


}

////smaze vsechny elektrony ve scene
////////////////////////////////////////////////////////
void deleteAllElektrons(list<Elektron*> *elektrons,SoSeparator * root) {
	vOut.clear();
	
	for (list<Elektron*>::iterator ci=elektrons->begin(); ci != elektrons->end(); ++ci) {
		vOut.push_back(*ci); //da do vOut
	}

	deleteElektrons(root,elektrons);
	elektrons->clear();


}


//////////smaze vsechny micro ve Scene
/////////////////////////////////////////////
void deleteAllMicros(list<Micro*> *micros,SoSeparator * root) {
	vOut_micros.clear();
	
	for (list<Micro*>::iterator ci=micros->begin(); ci != micros->end(); ++ci) {
		vOut_micros.push_back(*ci);
	}

	deleteMicros(root,micros);
	micros->clear();
}


//////////////////////////////////smaze vsehny Micro ve vektoru pro mazani
void deleteMicros(SoSeparator * r, list<Micro*> * micros) {

	 for (list<Micro*>::iterator ci=vOut_micros.begin(); ci != vOut_micros.end(); ++ci) {

	
		SoGroup * parent = getParent((*ci)->Croot,r);


		 //vytvoreni roztristeni elektronu na mensi castice
		 float cur_pos[3];
		 (*ci)->getPosition(cur_pos);

		 //zruseni elektronu
		 if (parent!=NULL) parent->removeChild((*ci)->Croot);
		 
	
	 } //end for

	 vOut_micros.clear();


}

///////uz prekrocil svoji delku vektoru?
////////////////////////////////////////////
bool Micro::getVecOver(){
	SbVec3f cur_vec(p_x-bPos_x,p_y-bPos_y,p_z-bPos_z); 
	float curlv =cur_vec.length(); //delka
	
	return (maxlv<curlv); //prekrocil?
}



///////////////////
