/*****************************************************************************\
*
* linac.cpp
*
* LINAC - simulation of using LINAC for acceleration of electron
*
* Authors: Ondrej Peterka (xpeter08 _AT stud.fit.vutbr.cz)
*          PCJohn (peciva _AT fit.vutbr.cz)
* Contributors:
*
* ----------------------------------------------------------------------------
*
* THIS SOFTWARE IS NOT COPYRIGHTED
*
* This source code is offered for use in the public domain.
* You may use, modify or distribute it freely.
*
* This source code is distributed in the hope that it will be useful but
* WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
* DISCLAIMED.  This includes but is not limited to warranties of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
*
* If you find the source code useful, authors will kindly welcome
* if you give them credit and keep their names with their source code,
* but do not feel to be forced to do so.
*
\*****************************************************************************/
#include <Inventor/SoInput.h>
#include <Inventor/nodes/SoCoordinate3.h>
#include <Inventor/nodes/SoIndexedTriangleStripSet.h>
#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoMaterial.h>
#include <Inventor/nodes/SoTranslation.h>
#include <Inventor/nodes/SoSphere.h>
#include <Inventor/nodes/SoScale.h>
#include <Inventor/nodes/SoTransform.h>
#include <Inventor/nodes/SoCylinder.h>
#include <Inventor/nodes/SoRotation.h>
#include <Inventor/nodes/SoShapeHints.h> 

#include <list>
#include <vector>
#include <math.h>
#include <iostream>

#include "scena.h"


 

std::vector<float> VecBoost; //vector meznich hodnot jednotlivych elektrod

const int mezera=3; //3; //mezera mezi elektrodami
float RightSizeOfRoura=-1;
float SizeOfPartOne=-1;
bool switchColor=true; //externi na zmenu barev u elektrod
bool oddColor=true; //lokalni pomocna na meneni barev
extern const float PozStartEl;  //startovni pozice elektronu v z-ose(definovana v scena.cpp)


std::list<SoMaterial *> VecElMat; //vektor materialu elektrod

/////////////vytvri nohu stojanu
SoSeparator * MakeLeg()
{


	SoSeparator * SepNoha1= new SoSeparator();



   SoCylinder *noha1H= new SoCylinder();
   noha1H->height=4.0;
   SepNoha1->addChild(noha1H);
    //1 koule :o)  
   

   SoTranslation * transKoule1 = new SoTranslation();
   transKoule1 ->translation.setValue(0.0,-2.0,0.0);
   SepNoha1->addChild(transKoule1);


      SoSphere *koule1= new SoSphere();
   koule1->radius=2.0;
   SepNoha1->addChild(koule1);

   //spodni cast nohy

   SoTransform * transNoha1L = new SoTransform();
   transNoha1L->translation.setValue(-10.0,0.0,0.0);
   transNoha1L->rotation.setValue(SbVec3f(0, 0, 1), ((const float) M_PI/2));
   SepNoha1->addChild(transNoha1L);

   SoCylinder * noha1L= new SoCylinder();
   noha1L->height=20.0;
   SepNoha1->addChild(noha1L);

   return SepNoha1;

}



////vytvori konzoli prebenchu
/////////////////////////////////////////////////////////////

SoSeparator * MakePrebench() {

  SoSeparator * prebench= new SoSeparator();  


  SoTranslation * transPreBench=new SoTranslation();
  transPreBench ->translation.setValue(0.0,0.0,-24.0);
  prebench->addChild(transPreBench);


 int i;
 for (i=1;i<10;i++) { //generuje prebench...    

   SoInput in;
   if (!in.openFile("matice.iv")) {
     printf("Can not open input file.\n");
     exit(-1);
   } 
   
   SoSeparator *matice = SoDB::readAll(&in); //nacitani z file
   if (matice == NULL) {
     printf("Can not read input file.\n");
     exit(-1);
   }
   prebench->addChild(matice);


   //posun

   SoTranslation * transFor = new SoTranslation();
   transFor ->translation.setValue(0.0,0.0,3.0);
   prebench->addChild(transFor);
 }//end for

return prebench;

}


//komora pro termoemisi
/////////////////////////////////////////////////////////////
SoSeparator * MakeKomora() {
	SoInput in;
	SoSeparator * SepKomora =  new SoSeparator();

  if (!in.openFile("stojan.iv")) {
    printf("Can not open input file.\n");
    exit(-1);
  }

  SoSeparator *komora = SoDB::readAll(&in);
  if (komora == NULL) {
    printf("Can not read input file.\n");
    exit(-1);
  }

  SoTranslation * transKomora=new SoTranslation();
  transKomora ->translation.setValue(-4.0,-4.0,-27.0);
  SepKomora->addChild(transKomora);

  SepKomora->addChild(komora);

  SoTranslation * transKatoda=new SoTranslation();
  transKatoda ->translation.setValue(4.0,4.0,-8.0);
  SepKomora->addChild(transKatoda);

  SoCylinder * katoda = new SoCylinder();
  katoda->radius=1;
  katoda->height=16;
  SepKomora->addChild(katoda);

     //------deska
  if (!in.openFile("deska.iv")) {
    printf("Can not open input file.\n");
    exit(-1);
  }

  SoSeparator *deska = SoDB::readAll(&in);
  if (deska == NULL) {
    printf("Can not read input file.\n");
    exit(-1);
  }

  SoTranslation * transDeska=new SoTranslation();
  transDeska ->translation.setValue(0.0,0.0,-8.0);
  SepKomora->addChild(transDeska);

  SepKomora->addChild(deska);


  

  return SepKomora;
    
}



///spoji vse predchozi dohromady
////////////////////////////////////////////////////////////
//vytvori prvni krabici ve ktere se emituje elektron
SoSeparator * MakeElGun() {

  SoSeparator * SepElGun=new SoSeparator();
SoInput in;
      //vytvori se komora s katodou
     SoSeparator * komora= MakeKomora();
  SepElGun->addChild(komora);



     //posune krabici tak aby byl stred diry v pocatku
     //FIXME: dat do iv
  SoTranslation * transElGun=new SoTranslation();
  transElGun ->translation.setValue(-10.0,-20.0,-26.0);
  SepElGun->addChild(transElGun);

  
  if (!in.openFile("ElGun_norm.iv")) {
    printf("Can not open input file.\n");
    exit(-1);
  }

  SoSeparator *ElGun = SoDB::readAll(&in);
  if (SepElGun == NULL) {
    printf("Can not read input file.\n");
    exit(-1);
  }
  SepElGun->addChild(ElGun);

///mala roura


  SoTranslation * transRoura=new SoTranslation();
  transRoura ->translation.setValue(10.0,20.0,12.0);
  SepElGun->addChild(transRoura);

  SoScale * scale_roura = new SoScale();
  scale_roura->scaleFactor.setValue(1.0,1.0,13.0);
  SepElGun->addChild(scale_roura);  

  if (!in.openFile("roura.iv")) {
    printf("Can not open input file.\n");
    exit(-1);
  }

  SoSeparator *roura_mala = SoDB::readAll(&in);
  if (SepElGun == NULL) {
    printf("Can not read input file.\n");
    exit(-1);
  }

  SepElGun->addChild(roura_mala);
///mala roura


  return SepElGun;


}


///stojan
/////////////////////////////////////////////////////////////////////
SoSeparator * MakeStojan() {
	SoInput in;

	      if (!in.openFile("stojan.iv")) {
     printf("Can not open input file.\n");
     exit(-1);
   }

   //Stojany
   SoSeparator *SepStojany= new SoSeparator();
   //stojan



     //drzak
   SoSeparator * SepDrzak = new SoSeparator();

   SoTranslation * transDrzak = new SoTranslation();
   transDrzak ->translation.setValue(-4.0,-4.0,8.0); //nastavi drzak aby sedel v zakrytu
   SepDrzak->addChild(transDrzak);
   
   SoSeparator *drzak = SoDB::readAll(&in); //FIXME: nacitat nebo kopirovat?
   if ( drzak == NULL) {
     printf("Can not read input file.\n");
     exit(-1);
   }
   SepDrzak->addChild(drzak);

    //noha1
        //vrchni cast nohy

   SoSeparator * SepNoha1= new SoSeparator();
   
   SoTransform * transNoha1H = new SoTransform();
   transNoha1H->translation.setValue(10.0,0.0,0.0);
   transNoha1H->rotation.setValue(SbVec3f(0, 0, 1), ((const float) M_PI/2));
   SepNoha1->addChild(transNoha1H);
   
   
   
   
   SoSeparator * Noha1= MakeLeg();
   SepNoha1->addChild(Noha1);


    //noha2


   SoSeparator * SepNoha2= new SoSeparator();
   
   SoTransform * transNoha2H = new SoTransform();
   transNoha2H->translation.setValue(-10.0,0.0,0.0);
   transNoha2H->rotation.setValue(SbVec3f(1, 1, 0), ((const float) M_PI));
   SepNoha2->addChild(transNoha2H);
   
   
   
   //spojeni
   SoSeparator * Noha2= MakeLeg();
   SepNoha2->addChild(Noha2);

    SepStojany->addChild(SepDrzak); 
    SepStojany->addChild(SepNoha1);
	SepStojany->addChild(SepNoha2);

	return SepStojany;
}

////vytvori urychlovaci trubici-jeji plast
/////////////////////////////////////////////////////////////////
SoSeparator * MakeRoura(int sizeLINAC) {
  SoInput in; 	
  SoSeparator * SepRoura=new SoSeparator();

  //posunuti o pulku celkove velikosti (roste totiz z nuly,...ale na obe tsrany)
  SoTranslation * transRoura=new SoTranslation();
  transRoura ->translation.setValue(0.0,0.0,(float) (sizeLINAC/2));
  SepRoura->addChild(transRoura);

  //zvetseni
  SoScale * scale_roura = new SoScale();
  scale_roura->scaleFactor.setValue(1.0,1.0,(float) (sizeLINAC/2));
  SepRoura->addChild(scale_roura);  

  if (!in.openFile("roura.iv")) {
    printf("Can not open input file.\n");
    exit(-1);
  }

  SoSeparator *roura = SoDB::readAll(&in);
  if (roura == NULL) {
    printf("Can not read input file.\n");
    exit(-1);
  }
  SepRoura->addChild(roura);
  
  return SepRoura;

}


//////////////////////////////////////////////////////////////
//vyrobi jednu valcovou elektrodu...patricne natocenou
SoSeparator * MakeOneEl(float size, bool nahore) {
	SoSeparator * SepEl= new SoSeparator();


    //barva

		SoMaterial * mat = new SoMaterial;
		mat->diffuseColor.setValue(SbColor(0.9f, 0.0f, 0.0f));
	    SepEl->addChild(mat);

		VecElMat.push_back(mat); //ulozi se ukazatel na material


	



    //posune aby spodek byl v nule
	SoTranslation * transEl = new SoTranslation();
	transEl->translation.setValue(0.0,0, (float) (size/2));
	SepEl->addChild(transEl);
  
	//otoci aby smerovala v ose z
	SoRotation * rotEl= new SoRotation();
	rotEl->rotation.setValue(SbVec3f(1,0,0), M_PI/2);
	SepEl->addChild(rotEl);


	SoCylinder * cyl= new SoCylinder();
	cyl->radius=2;
	cyl->height=size;
	cyl->removePart(SoCylinder::TOP);
	cyl->removePart(SoCylinder::BOTTOM);
	SepEl->addChild(cyl);

	//vnitrni plast
	SoCylinder * cylIn= new SoCylinder();
	cylIn->radius=1.7;
	cylIn->height=size;
	cylIn->removePart(SoCylinder::TOP);
	cylIn->removePart(SoCylinder::BOTTOM);
	SepEl->addChild(cylIn);

		//spojovaci koule
	  SoSeparator * SepSph = new SoSeparator();	  
	  SoSphere * sph= new SoSphere();
	  sph->radius=0.75;

	  SepEl->addChild(SepSph);
      SoTranslation * transSph = new SoTranslation();

	if (nahore) {
		//spojovaci koule se posune nahoru
	  
	  transSph->translation.setValue(5,0,-5 );
	  

	
	  SoRotation * rotVyvod= new SoRotation();
	  rotVyvod->rotation.setValue(SbVec3f(0,0,1), -(M_PI/2));
	  SepEl->addChild(rotVyvod);

	  SoTranslation * transVyvod = new SoTranslation();
	  transVyvod->translation.setValue(0,3,-3 );
	  SepEl->addChild(transVyvod);

      SoRotation * rotVyvod2= new SoRotation();
	  rotVyvod2->rotation.setValue(SbVec3f(1,0,0), -(M_PI/4));
	  SepEl->addChild(rotVyvod2);

	} else {

	  transSph->translation.setValue(5,0,5 );
	  

	  SoRotation * rotVyvod= new SoRotation();
	  rotVyvod->rotation.setValue(SbVec3f(0,0,1), (M_PI/2));
	  SepEl->addChild(rotVyvod);

	  SoTranslation * transVyvod = new SoTranslation();
	  transVyvod->translation.setValue(0,-3,3);
	  SepEl->addChild(transVyvod);

      SoRotation * rotVyvod2= new SoRotation();
	  rotVyvod2->rotation.setValue(SbVec3f(1,0,0), -(M_PI/4));
	  SepEl->addChild(rotVyvod2);

	}
       //vyvod elektriny
	SoCylinder *vyvod = new SoCylinder();
	vyvod->radius=0.2;
	vyvod->height=5;

	//vlozim do sepsph kouli :o)
	SepSph->addChild(transSph);
	SepSph->addChild(sph);


	SepEl->addChild(vyvod);

	return SepEl;

	


}

///////////////////////////////////////
//vytvori elktrody a vypocte delku linac-roury
SoSeparator * MakeElektrody(int sizeP1, int sizeLINAC, float & rightSize ){
	
    float Csize=(float) sizeP1;
    float suma=Csize;
	bool nahore=true;
	SoSeparator * SepElektrody = new SoSeparator();
	VecBoost.push_back(0.0); //dam nulu....je to prvni hodnota kde se bude zrychlovat

	int x = 0; //pro pomocne vypocty

	oddColor=switchColor;

	while (suma < sizeLINAC)  {
       SoGroup * GrEl = new SoGroup();	  //jedna elektroda se svym posunutim 
	   VecBoost.push_back(suma); 
 
	   //odecte se nutna mezera (pouze pro opticky dojem)...vpodtsate je mezera 
	   //tak mala ze by ji nemelo cenu vykreslovat
	   GrEl->addChild(MakeOneEl(Csize-mezera,nahore));

	   //posun   

	     SoTranslation * transEl=new SoTranslation();
         transEl ->translation.setValue(0.0,0.0,Csize);
         GrEl->addChild(transEl);      
	   
	   rightSize=suma; //tohle je skutecna velikost jakou bude mit linac(roura)
	  
	   x++;
	   SepElektrody->addChild(GrEl);
	   Csize*=(float) sqrt(x+1)/sqrt(x);
       suma+=(Csize);

	   nahore=!nahore;
       
	}

 
	return SepElektrody;
}





//////////////////////////////////////////////////////
//vytvori konzoli s tercikem...je umistena konci celeho LINACu
SoSeparator * MakeTercik() {
    int sizeObalka4=3;
	SoInput in;
	SoSeparator * SepTercik = new SoSeparator();

	    //------------------Desticka
  SoSeparator * SepDesticka = new SoSeparator();
  //poloha terciku je korigovana jednou jednotkou
  SoTranslation * transDesticka=new SoTranslation();
  transDesticka ->translation.setValue(0.0,0.0,(15+1.0));
  SepDesticka->addChild(transDesticka);

  	SoRotation * rotDesticka= new SoRotation();
	rotDesticka->rotation.setValue(SbVec3f(1,0,0), M_PI/2);
	SepDesticka->addChild(rotDesticka);
  
  SoCylinder * desticka = new SoCylinder();
  desticka->radius=3;


  SepDesticka->addChild(desticka);
  SepTercik->addChild(SepDesticka);

       //----------kruhy okolo obalky
  SoSeparator * SepKruhy = new SoSeparator();


  SoTranslation * transKruhy=new SoTranslation();
  transKruhy ->translation.setValue(0.0,-4.0,36.0);
  SepKruhy->addChild(transKruhy);

          //zvetseni
  SoScale * scaleKruhy = new SoScale();
  scaleKruhy->scaleFactor.setValue(2.0,3.0,1.0);
  SepKruhy->addChild(scaleKruhy);

  SoSeparator * kruhy= MakePrebench();
  SepKruhy->addChild(kruhy);  

  
  SepTercik->addChild(SepKruhy);
       //---------- posledni stojan
  SoSeparator * SepKonStojan = new SoSeparator();
  SoSeparator * konStojan=MakeStojan();

  SoTranslation * transKonStojan=new SoTranslation();
  transKonStojan ->translation.setValue( 0.0,0.0,(float) ((sizeObalka4)*(16+1))+5);
  SepKonStojan->addChild(transKonStojan);

  
  SepKonStojan->addChild(konStojan);
  SepTercik->addChild(SepKonStojan);


       //----------

  if (!in.openFile("stojan.iv")) {
    printf("Can not open input file.\n");
    exit(-1);
  }

  SoSeparator *obalka = SoDB::readAll(&in);
  if (obalka == NULL) {
    printf("Can not read input file.\n");
    exit(-1);
  }
  

    //posunuti o pulku celkove velikosti (roste totiz z nuly,...ale na obe tsrany)
  SoTranslation * transObalka=new SoTranslation();
  transObalka ->translation.setValue(-4.0,-4.0,(float) (sizeObalka4*16));
  SepTercik->addChild(transObalka);

  //zvetseni
  SoScale * scaleObalka = new SoScale();
  scaleObalka->scaleFactor.setValue(1.0,1.0,(float) (sizeObalka4));
  SepTercik->addChild(scaleObalka);  

  SepTercik->addChild(obalka); 

  

  return SepTercik;



}


////////////////////////////////////////////////////////////////
//vytvori vedeni okolo linacu
SoSeparator * MakeVedeni() {

	float prumerVedeni=0.5;
	SoSeparator * SepVedeni = new SoSeparator();

	SoSeparator * SepVedeni1 = new SoSeparator();


	SoCylinder * vedeniH = new SoCylinder();
	vedeniH->radius=prumerVedeni;
	vedeniH->height= RightSizeOfRoura-2;
	


	  //posunuti o pulku celkove velikosti (roste totiz z nuly,...ale na obe tsrany)
    SoTranslation * transVedeni1=new SoTranslation();
    transVedeni1 ->translation.setValue(5.0,5.0,(float) (RightSizeOfRoura/2));
    SepVedeni1->addChild(transVedeni1);

  	SoRotation * rotVedeni1= new SoRotation();
	rotVedeni1->rotation.setValue(SbVec3f(1,0,0), M_PI/2);
	SepVedeni1->addChild(rotVedeni1);


		//barva
		SoMaterial * mat = new SoMaterial;
		mat->diffuseColor.setValue(SbColor(0.9f, 0.0f, 0.0f));
	    SepVedeni1->addChild(mat);



	SepVedeni1->addChild(vedeniH);





	SoCylinder * vedeniD = new SoCylinder();
	vedeniD->radius=prumerVedeni;
	vedeniD->height= RightSizeOfRoura-2;


    SoTranslation * transVedeni2=new SoTranslation();
    transVedeni2 ->translation.setValue(0.0,0.0,10.0);
    SepVedeni1->addChild(transVedeni2);

		//barva
		SoMaterial * mat2 = new SoMaterial;
		mat2->diffuseColor.setValue(SbColor(0.0f, 0.9f, 0.0f));
	    SepVedeni1->addChild(mat2);

	SepVedeni1->addChild(vedeniD);


  
  SepVedeni->addChild(SepVedeni1);



  //aby se menili
  		VecElMat.push_front(mat2);
		VecElMat.push_front(mat);

  return SepVedeni;
}

//////////////////////////////////////////////////////
//vytvori generator a vedeni k vednei od LINACu

SoSeparator * MakeGenerator() {
	SoSeparator * SepGenerator = new SoSeparator();

	SoSeparator * SepKrabice = new SoSeparator();
    SoSeparator * SepVedeniH = new SoSeparator();
 
	//krabice
	SoCylinder * krabice = new SoCylinder();
	krabice->radius=10;
	krabice->height=50;

    SoTranslation * transKrabice=new SoTranslation();
    transKrabice ->translation.setValue(70.0,0.0,10.0);
    SepKrabice->addChild(transKrabice);

	SepKrabice->addChild(krabice);

	//horni vedeni
	SoCylinder * vedeniH = new SoCylinder();
	vedeniH->radius=0.5;
	vedeniH->height=60;

	SoTranslation * transVedeniH=new SoTranslation();
    transVedeniH ->translation.setValue(35.0,5.0,10.0);
    SepVedeniH->addChild(transVedeniH);

	

	SoRotation * rotVedeni1= new SoRotation();
	rotVedeni1->rotation.setValue(SbVec3f(0,0,1), M_PI/2);
	SepVedeniH->addChild(rotVedeni1);

			//barva
		SoMaterial * mat = new SoMaterial;
		mat->diffuseColor.setValue(SbColor(0.9f, 0.0f, 0.0f));
	    SepVedeniH->addChild(mat);

	SepVedeniH->addChild(vedeniH);

	//dolni
	SoCylinder * vedeniD = new SoCylinder();
	vedeniD->radius=0.5;
	vedeniD->height=60;

	SoTranslation * transVedeniD=new SoTranslation();
    transVedeniD ->translation.setValue(-10.0,0.0,0.0);
    SepVedeniH->addChild(transVedeniD);

			//barva
		SoMaterial * mat2 = new SoMaterial;
		mat2->diffuseColor.setValue(SbColor(0.9f, 0.0f, 0.0f));
	    SepVedeniH->addChild(mat2);

	SepVedeniH->addChild(vedeniD);

  //pospojovani
    SepGenerator->addChild(SepKrabice);
    SepGenerator->addChild(SepVedeniH);

	  //aby se menili
  		VecElMat.push_front(mat2);
		VecElMat.push_front(mat);

	return SepGenerator;

}

///////////////////////////////////////////////////////
//vytvori posloupnost stojanu
SoSeparator * MakeStojany() {
	int mezeraStojany=100;
	int suma=mezeraStojany;

	SoSeparator * SepStojany = new SoSeparator();

	SoGroup * GrStojan = new SoGroup();
	int i=0;
    if (RightSizeOfRoura>70) {
	
		while (suma<(RightSizeOfRoura-16)) {
   
			SoTranslation * transStojan=new SoTranslation();
      
			transStojan ->translation.setValue(0.0,0.0,mezeraStojany);
      
			GrStojan->addChild(transStojan);

	  
			SoSeparator * stojan = MakeStojan();
			GrStojan->addChild(stojan);
			suma+=mezeraStojany;
	   
	        i++;
		} //while
	} //if
	SepStojany->addChild(GrStojan);
   
	return SepStojany;

}



////////////////////////////////////////////////////////////////

//slozi vse dohromady....
SoSeparator * MakeLINAC(int sizeP1, int sizeLINAC ){

   SoSeparator * SepLinac=new SoSeparator( );
   SizeOfPartOne=sizeP1;


   //elgun
    SoSeparator * SepElGun=MakeElGun();
    SepLinac->addChild(SepElGun);

   //prebench
   SoSeparator * prebench= MakePrebench();
   SepLinac->addChild(prebench); 

  //  generator napeti
    SoSeparator * gen=MakeGenerator();  
    SepLinac->addChild(gen);  


  //roura plus elektrody
   float a; //pomocna promena pro skutecnou velikost rouru
   SoSeparator * Elektrody = new SoSeparator();


   Elektrody->addChild(MakeElektrody(sizeP1,sizeLINAC, a));

   SepLinac->addChild(Elektrody);
   RightSizeOfRoura=a;

   SoSeparator * Roura = MakeRoura(RightSizeOfRoura);
   SepLinac->addChild(Roura);

   //vedeni okolo linacu
   SoSeparator * vedeni=MakeVedeni();
   SepLinac->addChild(vedeni); 

   //vytvroi sadu stojanu
   SoSeparator * stojany=MakeStojany();  
   SepLinac->addChild(stojany); 



   //Konzole s tercikem
   SoSeparator * SepTerc = new SoSeparator();

   	SoTranslation * transTercik=new SoTranslation();
    transTercik ->translation.setValue(0.0,0.0,RightSizeOfRoura-1);
    SepTerc->addChild(transTercik);

	SoSeparator * tercik= MakeTercik();
    SepTerc->addChild(tercik);


	SepLinac->addChild(SepTerc);

   

	return SepLinac;

}

///////////////////////////////////////////////////