/*****************************************************************************\
*
* matice4.cpp
*
* LINAC - simulation of using LINAC for acceleration of electron
*
* Authors: Ondrej Peterka (xpeter08 _AT stud.fit.vutbr.cz)
*          PCJohn (peciva _AT fit.vutbr.cz)
* Contributors:
*
* ----------------------------------------------------------------------------
*
* THIS SOFTWARE IS NOT COPYRIGHTED
*
* This source code is offered for use in the public domain.
* You may use, modify or distribute it freely.
*
* This source code is distributed in the hope that it will be useful but
* WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
* DISCLAIMED.  This includes but is not limited to warranties of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
*
* If you find the source code useful, authors will kindly welcome
* if you give them credit and keep their names with their source code,
* but do not feel to be forced to do so.
*
\*****************************************************************************/

#define _WIN_32

#if defined(_WIN32) || defined(__WIN32__)
# pragma warning(disable:4275)  // Zaka varovn kompiltoru
# pragma warning(disable:4251)
# pragma comment(linker, "/entry:\"mainCRTStartup\"")  // Pro linker: Budeme linkovat s main,
                                                       // nikoliv WinMain, jak je na Windows zvykem
#endif


#if defined(_WIN32) || defined(__WIN32__)
# include <Inventor/Win/SoWin.h>
# include <Inventor/Win/viewers/SoWinExaminerViewer.h>
#else
# include <Inventor/Qt/SoQt.h>
# include <Inventor/Qt/viewers/SoQtExaminerViewer.h>
#endif
#include <iostream>
#include <list>
#include <vector>

#include "linac.h"
#include "scena.h"
#include "menu.h"
#include "matice4.h"
#include "clipPlane.h"
#include "elektron.h"

#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoSphere.h>

#include <Inventor/Win/devices/SoWinKeyboard.h> 
#include <Inventor/sensors/SoTimerSensor.h> 
#include <Inventor/nodes/SoMaterial.h>


#if defined(_WIN32) || defined(__WIN32__)
  SoWinExaminerViewer *viewerX;
#else
  SoQtExaminerViewer *viewerX;
#endif


  SoSeparator * Sroot; //root
  SoSeparator * Scene; //podkoren sceny
  extern  SoTimerSensor * tSensor; //na blikani elelktrod
  extern std::list<SoMaterial *> VecElMat; //vector materialu elektrod
  extern std::vector<float> VecBoost; //vector meznich hodnot jednotlivych elektrod
  extern Elektron * pickedElectron;


 extern  float setlengthLinac;
 extern float setlengthFirstLinac;
  extern double setfrekvence;
  extern float setMAXVolts;


  float lengthLinac = 300;
  float lengthFirstLinac = 10;
  double frekvence = 93773.66;
  float MAXVolts=10;

/////pro vykresleni noveho linacu podle zadanych parametru
//////////////////////////////////////////////////////////////////
void redrawLINAC(){

	if (tSensor!=NULL && tSensor->isScheduled()) tSensor->unschedule();
	if (tSensor!=NULL) delete tSensor;
	tSensor=NULL;
	VecElMat.clear(); //mazani vektoru ktere se musi vytvorit znovu
	VecBoost.clear();

	if (pickedElectron != NULL) {
		delete pickedElectron;
		pickedElectron = NULL;
	}


	    lengthLinac = setlengthLinac;
        lengthFirstLinac = setlengthFirstLinac;
        frekvence = setfrekvence;
        MAXVolts = setMAXVolts;



	Scene->replaceChild(1,MakeLINAC(lengthFirstLinac,lengthLinac));

	

}



///////////////////////////////////////////////////
int main(int argc, char **argv)
{
  // Inicializuj Inventor
#if defined(_WIN32) || defined(__WIN32__)
  HWND window = SoWin::init(argv[0]);
#else
  QWidget *window = SoQt::init(argv[0]);
#endif
  if (window == NULL) exit(1);

  // Vytvo okno prohlee
#if defined(_WIN32) || defined(__WIN32__)
  viewerX = new SoWinExaminerViewer(window);
#else
  viewerX = new SoQtExaminerViewer(window);
#endif

///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
   

  Sroot =new SoSeparator();
 // Sroot->ref();
  viewerX->setSceneGraph(Sroot);
 


  Sroot->insertChild(new SoGroup(),0);

  Scene= new SoSeparator();

  SoClipPlane * clip = makeClipPlane(viewerX);
  SoSeparator * LINAC=MakeLINAC(lengthFirstLinac,lengthLinac);  


  Scene->addChild(clip);
  Scene->addChild(LINAC);

	//vlozi se model linacu
  Sroot->addChild(Scene);   

  //vyvtori se elektorn
  createElektrons(Sroot);  

  //create Menu
  SoSeparator * Menu = MakeMenu(viewerX);
  Sroot->addChild(Menu);  



////////////////////////////////////////////////////////
////////////////////////////////////////////////////////
////////////////////////////////////////////////////////



  viewerX->viewAll();
  viewerX->setTitle("SimpleModel 3");
  viewerX->show();

  // Zobraz okna a rozje renderovac smyku
#if defined(_WIN32) || defined(__WIN32__)
  SoWin::show(window);
  SoWin::mainLoop();
#else
  SoQt::show(window);
  SoQt::mainLoop();
#endif

  // Uvolni pam
  delete viewerX;
//  Sroot->unref();

  return 0;
}
