/*****************************************************************************\
*
* menu.cpp
*
* LINAC - simulation of using LINAC for acceleration of electron
*
* Authors: Ondrej Peterka (xpeter08 _AT stud.fit.vutbr.cz)
*          PCJohn (peciva _AT fit.vutbr.cz)
* Contributors:
*
* ----------------------------------------------------------------------------
*
* THIS SOFTWARE IS NOT COPYRIGHTED
*
* This source code is offered for use in the public domain.
* You may use, modify or distribute it freely.
*
* This source code is distributed in the hope that it will be useful but
* WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
* DISCLAIMED.  This includes but is not limited to warranties of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
*
* If you find the source code useful, authors will kindly welcome
* if you give them credit and keep their names with their source code,
* but do not feel to be forced to do so.
*
\*****************************************************************************/

#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoOrthographicCamera.h>
#include <Inventor/nodes/SoPerspectiveCamera.h>
#include <Inventor/nodes/SoSphere.h> 
#include <Inventor/nodes/SoText2.h>
#include <Inventor/nodes/SoFont.h>
#include <Inventor/nodes/SoMaterial.h>
#include <Inventor/Win/viewers/SoWinExaminerViewer.h>
#include <Inventor/nodes/SoTranslation.h>

#include <Inventor/nodes/SoEventCallback.h>
#include <Inventor/SoPickedPoint.h>
#include <Inventor/actions/SoRayPickAction.h>
#include <Inventor/events/SoMouseButtonEvent.h>

#include <Inventor/sensors/SoTimerSensor.h>
#include <Inventor/nodes/SoCallback.h> 

#include <iostream>

#include "scena.h"
#include "elektron.h"

//bool stopFlag=false;
bool bReset=false;
bool bRedraw=false;

extern SoOneShotSensor *sensor;
extern SbTime lastTimeTick;
extern SoTimerSensor * tSensor;
extern float velSim;


extern  float lengthLinac;
extern  float lengthFirstLinac;
extern  double frekvence;
extern  float MAXVolts;

extern double pocRychlost;



float setlengthLinac = lengthLinac;
float setlengthFirstLinac = lengthFirstLinac;
double setfrekvence = frekvence;
float setMAXVolts = MAXVolts;

int rozmer = 1; //rozmer pro nastavovani atributu 1,10,100,1000,...

int korekce =1; //korekce pro rychlost simulace




SoOrthographicCamera * camera; //ortho

 SoText2 * enOut; //vypis energie
 SoText2 * velOut; //vypis rychlodti zvoleneho elektornu
 SoText2 * velsimMenu; //vypis zrychleni simulace
 SoText2 * lengthMenu; //vypis delky linacu		
 SoText2 * lengthFirstMenu; //vypis delky 1. elektrody
 SoText2 * frequencyMenu; //vypis frekvence
 SoText2 * voltageMenu;  //vypis napeti

//ukazatele na vyjadreni prepinace romeru
 SoText2 * Troz; 
 SoText2 * Hroz;
 SoText2 * Droz;
 SoText2 * Proz;
 SoText2 * DTroz;
 SoText2 * HTroz;
//ukazatel na reprezentaci infoline v menu
 SoText2 * infoLine;

 SoText2 * btnCP; //pro prepis nazvu tlacitka clip plane
 SoText2 * btnITCP; //--""-- pro IT clip plane

 SoText2 * btnStopStart; //--""-- pro start/stop 

bool bClipPlane=true;
bool bITClipPlane=false;


float moveMenu = -0.06;//posun mezi polozkami
int vsim = 5; //urcuje zrychleni v menu

SbTime lastTimeBase=SbTime::zero();

/////callback fce aby se mnu nerozjizdelo ... od Ing.Jana Pecivy
///////////////////////////////////////////
static void
overlayViewportCB(void *, SoAction *action)
{

	   if (action->isOfType(SoGLRenderAction::getClassTypeId())) {
       const SbViewportRegion &vr =
                  ((SoGLRenderAction*)action)->getViewportRegion();
       float pointSize = 2.f / 357; // 2.f is default camera view height and 357 default window height
                                    // Dividing 2.f by 357 we receive size of one point on the screen.
       camera->height = pointSize * vr.getWindowSize()[1]; // we will receive the same point resolution on each window size
       float posX = (float(vr.getWindowSize()[0]) / 2.f - 20) * pointSize;
       float posY = -camera->height.getValue() / 2.f + pointSize * 20.f;
       camera->position.setValue(posX, posY, 1.f);

   }
  }

//osetri zastaveni animace/rozjeti animace
///////////////////////////////////////////////////
void handleStopStart(bool stopFlag) {
		if (stopFlag) {
            
			sensor->unschedule();
			if (tSensor!=NULL && tSensor->isScheduled()) {
			  lastTimeBase = tSensor->getBaseTime();			  
			  tSensor->unschedule();
			}
		} else {
		
			lastTimeTick=SbTime::zero();

			sensor->schedule();
			if (tSensor!=NULL && !tSensor->isScheduled()) {
			  tSensor->setBaseTime(lastTimeBase);
			  tSensor->schedule();
			}
		}
}

/////////////////////////////prenastavi rychlost simulace 
void setVelsim() {
	if (setMAXVolts >1000000) {
		korekce=100;
		velSim=100 * 1e5;
		
		return;
	}
	if (setMAXVolts >100000) {
		korekce=100;
		velSim=100 * 1e5;
		return;
	}
	if (setMAXVolts >10000) {
		korekce=10;
		velSim=10 * 1e5;
		return;
	}
	if (setMAXVolts >1000) {
		korekce=10;
		velSim=10 *1e5;
		return;
	}

		korekce=1;
		velSim=1e5;
		return;


}




//kontroluje zdali je nastavene vse pro vytvoreni linacu tak jak ma byt
//////////////////////////////////////////////////////////////////////
bool kontrolaOK() {

	if (setlengthFirstLinac > setlengthLinac) {

		infoLine->string = "Length of LINAC has to be bigger than its first electrode.";

		return false;
	}


	infoLine->string = "New LINAC created.";
	return true;


}

//////callback fce pro mys
//////////////////////////////
static void
event_cb(void * ud, SoEventCallback * n)
{
  const SoMouseButtonEvent * mbe = (SoMouseButtonEvent *)n->getEvent();

  if (mbe->getButton() == SoMouseButtonEvent::BUTTON1 &&
      mbe->getState() == SoButtonEvent::DOWN) {
    SoWinExaminerViewer * viewer = (SoWinExaminerViewer *)ud;	
	
	SoRayPickAction rp(viewer->getViewportRegion());
    rp.setPoint(mbe->getPosition());	
    rp.apply(viewer->getSceneManager()->getSceneGraph());

    SoPickedPoint * point = rp.getPickedPoint();
    if (point == NULL) { //nic nebylo vybrano      
      return;
    }

    

	SoNode * MyNode = point->getPath()->getTail(); //jaky objekt se vybral

	SoText2  * pickedBtn; //sem si ulozim vybrane tlacitko/elektron
	
	n->setHandled();	

	if (MyNode->getTypeId() == SoSphere::getClassTypeId()){ //je to elektron nebo micro
		SoSphere * pickedSphere = (SoSphere *) MyNode;		//bezpecne pretypovani
		findElektron(pickedSphere);		//najdi elektorn..nemusi existovat
		return;
	} else if (MyNode->getTypeId() == SoText2::getClassTypeId()){ //je to tlacitko
		pickedBtn = (SoText2 *) MyNode;
	} else return;
	
	

	

   ///osetreni buttonu
	//-------------------------------------------------------------------------

	if (!strcmp(pickedBtn->string.getValues(0)->getString(), "Fire electron")){		   
		create1Elektron((SoSeparator *) viewer->getSceneGraph());
	}


	if (!strcmp(pickedBtn->string.getValues(0)->getString(), "Stop animation")){
		handleStopStart(true);
		btnStopStart->string = "Start animation"; //prenastaveni textu
		return;
	}

	if (!strcmp(pickedBtn->string.getValues(0)->getString(), "Start animation")){
			
		handleStopStart(false);
		btnStopStart->string = "Stop animation";
		return;
	}



	if (!strcmp(pickedBtn->string.getValues(0)->getString(), "Reset")){	

	   bReset=true;		
	}


	if (!strcmp(pickedBtn->string.getValues(0)->getString(), "ON clipPlane")){		
	 
	   
	   btnCP->string = "OFF clipPlane";

	   bClipPlane=true;	   
	   SbVec3f i=(SbVec3f) viewer->getCamera()->position.getValue();
		   i[2]+0.000001; //vyvola zmenu ihned
	    viewer->getCamera()->position.setValue(i);
		return;
		
	}


	if (!strcmp(pickedBtn->string.getValues(0)->getString(), "OFF clipPlane")){		   
	   
	   btnCP->string = "ON clipPlane";

	   bClipPlane = false;
	   
	   //vyvola se zmena
	   SbVec3f i=(SbVec3f) viewer->getCamera()->position.getValue();
		   i[2]+0.000001;
	    viewer->getCamera()->position.setValue(i);
		return;
		
	}




	if (!strcmp(pickedBtn->string.getValues(0)->getString(), "ON IT clipPlane") )
	{		
	  
		btnITCP->string = "OFF IT clipPlane";
	   bITClipPlane=true;
	   

	   //vyvola se zmena
	   SbVec3f i=(SbVec3f) viewer->getCamera()->position.getValue();
	   i[2]+0.000001;
	    viewer->getCamera()->position.setValue(i);
		return;
		
	}


	if (!strcmp(pickedBtn->string.getValues(0)->getString(), "OFF IT clipPlane") )
	{		
	  
		btnITCP->string = "ON IT clipPlane";
	   bITClipPlane=false;
	   

	   //vyvola se zmena
	   SbVec3f i=(SbVec3f) viewer->getCamera()->position.getValue();
	   i[2]+0.000001;
	    viewer->getCamera()->position.setValue(i);
		return;
		
	}


	if (!strcmp(pickedBtn->string.getValues(0)->getString(), "Redraw")){
		

		if	(!kontrolaOK()) {
			return;
		}

		pocRychlost = velSim/10;

        bRedraw = true; //priznak
		
	}

	//jen pro debug
	if (!strcmp(pickedBtn->string.getValues(0)->getString(), "Help")){		
   
		
	}

	//zrychleni simulace
	if (!strcmp(pickedBtn->string.getValues(0)->getString(), "sim up")){
	
		
      if ((vsim+rozmer)<10) {
		velSim-=12000 * (korekce*rozmer);
		velsimMenu->string = (vsim+=rozmer);
		//nastaveni blikani elektrod
		if (tSensor !=NULL) tSensor->setInterval(velSim/(2*frekvence));
	  }
	}
	

	if (!strcmp(pickedBtn->string.getValues(0)->getString(), "down")){		
        std::cout<<"aaaa\n";
		if ((vsim-rozmer)>-99) {
		
			velSim+=12000*(korekce*rozmer);
			velsimMenu->string = (vsim-=rozmer);
			if (tSensor !=NULL) tSensor->setInterval(velSim/(2*frekvence));
		}
		
	}

	//length all

	if (!strcmp(pickedBtn->string.getValues(0)->getString(), "linac length up")){
	
		

			setlengthLinac += rozmer; //zvetseni linacu
			if (setlengthLinac > 10000) setlengthLinac = 10000;
			lengthMenu->string = setlengthLinac;

	}
	
	if (!strcmp(pickedBtn->string.getValues(0)->getString(), "down ")){		
        

		
			setlengthLinac -= rozmer; //zmenseni linacu
			if (setlengthLinac < 200) setlengthLinac = 200;
			lengthMenu->string = setlengthLinac;

		
	}


	//////////////////////////length first

	if (!strcmp(pickedBtn->string.getValues(0)->getString(), "first length up")){


			setlengthFirstLinac += rozmer;
			if (setlengthFirstLinac > 1000) setlengthFirstLinac = 1000;
			lengthFirstMenu->string = setlengthFirstLinac;
	
		

	}
	
	if (!strcmp(pickedBtn->string.getValues(0)->getString(), "down  ")){		
        

			setlengthFirstLinac -= rozmer;
			if (setlengthFirstLinac < 10) setlengthFirstLinac = 10;
			lengthFirstMenu->string = setlengthFirstLinac;

		
	}

	/////////////////////////////////frequency

	if (!strcmp(pickedBtn->string.getValues(0)->getString(), "frequency up")){
	
		

			setfrekvence += rozmer;
            if (setfrekvence > 10000000)   setfrekvence = 10000000;
			frequencyMenu->string = setfrekvence;

			//projevi se ihned
			if ((velSim/(2*frekvence))>0.1) //min coin nezvlada
			 if (tSensor !=NULL)   tSensor->setInterval(velSim/(2*frekvence));
			frekvence = setfrekvence;

	}
	
	if (!strcmp(pickedBtn->string.getValues(0)->getString(), "down   ")){		
        

		
			setfrekvence -= rozmer;
			if (setfrekvence < 1000)   setfrekvence = 1000;
			frequencyMenu->string = setfrekvence;

			//projevi se ihned
		//	if ((velSim/(2*frekvence))>0.1) //min coin nezvlada
		 if (tSensor !=NULL) 	   tSensor->setInterval(velSim/(2*frekvence));
			frekvence = setfrekvence;

		
	}

	//////////////////////////////voltage

	if (!strcmp(pickedBtn->string.getValues(0)->getString(), "voltage up")){
	
	   	

		setMAXVolts +=rozmer;
		if (setMAXVolts > 50000)  setMAXVolts = 50000; //vic jak 1000 nepujde
		voltageMenu->string = setMAXVolts;

    	//projevi se ihned
		MAXVolts = setMAXVolts;

	}
	
	if (!strcmp(pickedBtn->string.getValues(0)->getString(), "down    ")){		
      
	  	

		setMAXVolts -= rozmer;
		if (setMAXVolts < 10)  setMAXVolts = 10; //min jak deset nepujde
		voltageMenu->string = setMAXVolts;

		//projevi se ihned
		MAXVolts = setMAXVolts;


		
	}


       //---------------------prepinace---------------------
	if (!strcmp(pickedBtn->string.getValues(0)->getString(), "p")){
	
		rozmer = 1;
		Proz->string = "P";
		Hroz->string = "h";
		Troz->string = "t";
		Droz->string = "d";
		DTroz->string = "dt";
		HTroz->string = "ht";

	}
	
	if (!strcmp(pickedBtn->string.getValues(0)->getString(), "d")){		
        

		rozmer = 10;
		Proz->string = "p";
		Hroz->string = "h";
		Troz->string = "t";
		Droz->string = "D";
		DTroz->string = "dt";
		HTroz->string = "ht";
		
	}

	if (!strcmp(pickedBtn->string.getValues(0)->getString(), "h")){
	
		rozmer = 100;
		Proz->string = "p";
		Hroz->string = "H";
		Troz->string = "t";
		Droz->string = "d";
		DTroz->string = "dt";
		HTroz->string = "ht";
		

	}
	
	if (!strcmp(pickedBtn->string.getValues(0)->getString(), "t")){		
		rozmer =1000;
		Proz->string = "p";
		Hroz->string = "h";
		Troz->string = "T";
		Droz->string = "d";
		DTroz->string = "dt";
		HTroz->string = "ht";
		
	}

	if (!strcmp(pickedBtn->string.getValues(0)->getString(), "dt")){		
		rozmer =10000;
		Proz->string = "p";
		Hroz->string = "h";
		Troz->string = "t";
		Droz->string = "d";
		DTroz->string = "DT";
		HTroz->string = "ht";
		
	}

	if (!strcmp(pickedBtn->string.getValues(0)->getString(), "ht")){		
		rozmer =100000;
		Proz->string = "p";
		Hroz->string = "h";
		Troz->string = "t";
		Droz->string = "d";
		DTroz->string = "dt";
		HTroz->string = "HT";
		
	}


  }


}



//vytvoreni samotneho menu
////////////////////////////////////

SoSeparator * MakeMenu(SoWinExaminerViewer * viewer){
	SoSeparator * MyMenu = new SoSeparator;




	    // Add a callback node which will set the viewport
    SoCallback *overlayCB = new SoCallback;
    overlayCB->setCallback(overlayViewportCB);
    MyMenu->addChild(overlayCB);


	camera = new SoOrthographicCamera;

	MyMenu->addChild(camera);



    SoEventCallback * ecb = new SoEventCallback; //callback udalosti pro mys
    ecb->addEventCallback(SoMouseButtonEvent::getClassTypeId(), event_cb, viewer);
    MyMenu->addChild(ecb);

	SoMaterial *material = new SoMaterial;
	material->diffuseColor.setValue(SbColor(0.1f, 0.3f, 0.9f));
	MyMenu->addChild(material);

	SoFont *myFont = new SoFont;
    myFont->name.setValue("Helvetica");
    myFont->size.setValue(10);
    MyMenu->addChild(myFont);  
	
	//infoLine    
	SoText2 * btn0 = new SoText2();
	btn0->string = "InfoLine: ";
	MyMenu->addChild(btn0);


	SoTranslation * transBtn1 = new SoTranslation;
	transBtn1->translation.setValue(0.4,0.0,0.0);
	MyMenu->addChild(transBtn1);

    
	infoLine = new SoText2();
	infoLine->string = "";
	MyMenu->addChild(infoLine);


   //Fire elcetron
	SoTranslation * transBtn = new SoTranslation;
	transBtn->translation.setValue(-0.4,moveMenu,0.0);
	MyMenu->addChild(transBtn);

    
	SoText2 * btn1 = new SoText2();
	btn1->string = "Fire electron";
	MyMenu->addChild(btn1);

	//stop
	SoTranslation * transBtn2 = new SoTranslation;
	transBtn2->translation.setValue(0.0,moveMenu,0.0);
	MyMenu->addChild(transBtn2);

	btnStopStart = new SoText2();
	btnStopStart->string = "Stop animation";
	MyMenu->addChild(btnStopStart);

    //RESET
	SoTranslation * transBtn3 = new SoTranslation;
	transBtn3->translation.setValue(0.0,moveMenu,0.0);
	MyMenu->addChild(transBtn3);

	SoText2 * btn3 = new SoText2();
	btn3->string = "Reset";
	MyMenu->addChild(btn3);

	//ON/OFF clipPlane
	SoTranslation * transBtn4 = new SoTranslation;
	transBtn4->translation.setValue(0.0,moveMenu,0.0);
	MyMenu->addChild(transBtn4);

	btnCP = new SoText2();
	btnCP->string = "OFF clipPlane";
	MyMenu->addChild(btnCP);

	//ON/OFF IT clipPlane
	SoTranslation * transBtn5 = new SoTranslation;
	transBtn5->translation.setValue(0.0,moveMenu,0.0);
	MyMenu->addChild(transBtn5);

	btnITCP = new SoText2();
	btnITCP->string = "ON IT clipPlane";
	MyMenu->addChild(btnITCP);

	//ON/OFF IT clipPlane
	SoTranslation * transBtn12 = new SoTranslation;
	transBtn12->translation.setValue(0.0,moveMenu,0.0);
	MyMenu->addChild(transBtn12);

	SoText2 * btn12 = new SoText2();
	btn12->string = "Redraw";
	MyMenu->addChild(btn12);

/* //jen pro debug
	//Help
	SoTranslation * transBtn13 = new SoTranslation;
	transBtn13->translation.setValue(0.0,moveMenu,0.0);
	MyMenu->addChild(transBtn13);

	SoText2 * btn13 = new SoText2();
	btn13->string = "Help";
	MyMenu->addChild(btn13);
*/
	
	//////////////////////////uz nejsou buttony ...vypis elektronu

		//Velocity
	SoTranslation * transBtn6 = new SoTranslation;
	transBtn6->translation.setValue(0.0,moveMenu,0.0);
	MyMenu->addChild(transBtn6);

	SoText2 * btn6 = new SoText2();
	btn6->string = "Velocity: [m/s]";
	MyMenu->addChild(btn6);

		//Energy
	SoTranslation * transBtn7 = new SoTranslation;
	transBtn7->translation.setValue(0.0,moveMenu,0.0);
	MyMenu->addChild(transBtn7);

	SoText2 * btn7 = new SoText2();
	btn7->string = "Energy [eV]: ";
	MyMenu->addChild(btn7);



	//////////////////////

	SoTranslation * transBtn8 = new SoTranslation;
	transBtn8->translation.setValue(0.6,-moveMenu,0.0);
	MyMenu->addChild(transBtn8);

	velOut = new SoText2();
	velOut->string = "";
	MyMenu->addChild(velOut);

		//Energy
	SoTranslation * transBtn9 = new SoTranslation;
	transBtn9->translation.setValue(0.0,moveMenu,0.0);
	MyMenu->addChild(transBtn9);

	enOut = new SoText2();
	enOut->string = "";
	MyMenu->addChild(enOut);

    //////////////////////////////zrychlovani, zpomlaovani simulace
    //++
	SoTranslation * transBtn20 = new SoTranslation;
	transBtn20->translation.setValue(0.4,0.5,0.0);
	MyMenu->addChild(transBtn20);

	SoText2 * btn20 = new SoText2();
	btn20->string = "sim up";
	MyMenu->addChild(btn20);

    // udaj o zrychleni
	SoTranslation * transBtn22 = new SoTranslation;
	transBtn22->translation.setValue(0.2,0.0,0.0);
	MyMenu->addChild(transBtn22);

	velsimMenu = new SoText2();
	velsimMenu->string = vsim;
	MyMenu->addChild(velsimMenu);

	//--
	SoTranslation * transBtn21 = new SoTranslation;
	transBtn21->translation.setValue(0.1,0.0,0.0);
	MyMenu->addChild(transBtn21);

	SoText2 * btn21 = new SoText2();
	btn21->string = "down";
	MyMenu->addChild(btn21);

	    //////////////////////////////delka linacu
    //++
	SoTranslation * transBtn24 = new SoTranslation;
	transBtn24->translation.setValue(-0.3,moveMenu,0.0);
	MyMenu->addChild(transBtn24);

	SoText2 * btn24 = new SoText2();
	btn24->string = "linac length up";
	MyMenu->addChild(btn24);

    // udaj o zrychleni
	SoTranslation * transBtn25 = new SoTranslation;
	transBtn25->translation.setValue(0.5,0.0,0.0);
	MyMenu->addChild(transBtn25);

	lengthMenu = new SoText2();
	lengthMenu->string = setlengthLinac; //<---
	MyMenu->addChild(lengthMenu);

	//--
	SoTranslation * transBtn26 = new SoTranslation;
	transBtn26->translation.setValue(0.3,0.0,0.0);
	MyMenu->addChild(transBtn26);

	SoText2 * btn26 = new SoText2();
	btn26->string = "down ";
	MyMenu->addChild(btn26);


		    //////////////////////////////delka prvni elektrody linacu
    //++
	SoTranslation * transBtn27 = new SoTranslation;
	transBtn27->translation.setValue(-0.8,moveMenu,0.0);
	MyMenu->addChild(transBtn27);

	SoText2 * btn27 = new SoText2();
	btn27->string = "first length up";
	MyMenu->addChild(btn27);

    // udaj o zrychleni
	SoTranslation * transBtn28 = new SoTranslation;
	transBtn28->translation.setValue(0.5,0.0,0.0);
	MyMenu->addChild(transBtn28);

	lengthFirstMenu = new SoText2();
	lengthFirstMenu->string = setlengthFirstLinac; //<---
	MyMenu->addChild(lengthFirstMenu);

	//--
	SoTranslation * transBtn29 = new SoTranslation;
	transBtn29->translation.setValue(0.3,0.0,0.0);
	MyMenu->addChild(transBtn29);

	SoText2 * btn29 = new SoText2();
	btn29->string = "down  ";
	MyMenu->addChild(btn29);


			    //////////////////////////////frekvence
    //++
	SoTranslation * transBtn30 = new SoTranslation;
	transBtn30->translation.setValue(-0.8,moveMenu,0.0);
	MyMenu->addChild(transBtn30);

	SoText2 * btn30 = new SoText2();
	btn30->string = "frequency up";
	MyMenu->addChild(btn30);

    // udaj o zrychleni
	SoTranslation * transBtn31 = new SoTranslation;
	transBtn31->translation.setValue(0.5,0.0,0.0);
	MyMenu->addChild(transBtn31);

	frequencyMenu = new SoText2();
	frequencyMenu->string = setfrekvence; 
	MyMenu->addChild(frequencyMenu);

	//--
	SoTranslation * transBtn32 = new SoTranslation;
	transBtn32->translation.setValue(0.3,0.0,0.0);
	MyMenu->addChild(transBtn29);

	SoText2 * btn32 = new SoText2();
	btn32->string = "down   ";
	MyMenu->addChild(btn32);




	
	
	
	//////////////////////////////Voltage
    //++
	SoTranslation * transBtn33 = new SoTranslation;
	transBtn33->translation.setValue(-0.8,moveMenu,0.0);
	MyMenu->addChild(transBtn33);

	SoText2 * btn33 = new SoText2();
	btn33->string = "voltage up";
	MyMenu->addChild(btn33);

    // udaj 
	SoTranslation * transBtn34 = new SoTranslation;
	transBtn34->translation.setValue(0.5,0.0,0.0);
	MyMenu->addChild(transBtn34);

	voltageMenu = new SoText2();
	voltageMenu->string = setMAXVolts; //<---
	MyMenu->addChild(voltageMenu);

	//--
	SoTranslation * transBtn35 = new SoTranslation;
	transBtn35->translation.setValue(0.3,0.0,0.0);
	MyMenu->addChild(transBtn35);

	SoText2 * btn35 = new SoText2();
	btn35->string = "down    ";
	MyMenu->addChild(btn35);




		////////////////////////nastaveni rozmeru -prepinace

		SoTranslation * transBtn40 = new SoTranslation;
	transBtn40->translation.setValue(-0.8,moveMenu,0.0);
	MyMenu->addChild(transBtn40);

	Proz = new SoText2();
	Proz->string = "P";
	MyMenu->addChild(Proz);

	
		//--
	SoTranslation * transBtn41 = new SoTranslation;
	transBtn41->translation.setValue(0.1,0.0,0.0);
	MyMenu->addChild(transBtn41);

	Droz = new SoText2();
	Droz->string = "d";
	MyMenu->addChild(Droz);


	SoTranslation * transBtn42 = new SoTranslation;
	transBtn42->translation.setValue(0.1,0.0,0.0);
	MyMenu->addChild(transBtn42);

	Hroz = new SoText2();
	Hroz->string = "h";
	MyMenu->addChild(Hroz);

	
		//--
	SoTranslation * transBtn43 = new SoTranslation;
	transBtn43->translation.setValue(0.1,0.0,0.0);
	MyMenu->addChild(transBtn43);

	Troz = new SoText2();
	Troz->string = "t";
	MyMenu->addChild(Troz);

	//////////
		SoTranslation * transBtn44 = new SoTranslation;
	transBtn44->translation.setValue(0.1,0.0,0.0);
	MyMenu->addChild(transBtn44);

	DTroz = new SoText2();
	DTroz->string = "dt";
	MyMenu->addChild(DTroz);

	/////////
	SoTranslation * transBtn45 = new SoTranslation;
	transBtn45->translation.setValue(0.1,0.0,0.0);
	MyMenu->addChild(transBtn45);

	HTroz = new SoText2();
	HTroz->string = "ht";
	MyMenu->addChild(HTroz);


	return MyMenu;
}