/*****************************************************************************\
*
* scena.cpp
*
* LINAC - simulation of using LINAC for acceleration of electron
*
* Authors: Ondrej Peterka (xpeter08 _AT stud.fit.vutbr.cz)
*          PCJohn (peciva _AT fit.vutbr.cz)
* Contributors:
*
* ----------------------------------------------------------------------------
*
* THIS SOFTWARE IS NOT COPYRIGHTED
*
* This source code is offered for use in the public domain.
* You may use, modify or distribute it freely.
*
* This source code is distributed in the hope that it will be useful but
* WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
* DISCLAIMED.  This includes but is not limited to warranties of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
*
* If you find the source code useful, authors will kindly welcome
* if you give them credit and keep their names with their source code,
* but do not feel to be forced to do so.
*
\*****************************************************************************/

#include <Inventor/sensors/SoOneShotSensor.h>
#include <Inventor/sensors/SoTimerSensor.h>
#include <iostream>
#include <list>
#include "elektron.h"
#include "matice4.h"
//SoSeparator *sceneRoot;			// koren sceny
using namespace std;
typedef list<Elektron*> TElektrons;
typedef list<Micro*> TMicros;

const double PozStartEl=-0.00000001; //kde elektron zacina(melo by licovat s katodou)
double pocRychlost = 100000;  

#include <stdlib.h>
#include <time.h>

SbTime lastTimeTick = SbTime::zero();	// cas minuleho pocitani pozic objektu

SoOneShotSensor *sensor;		// senzor na pohyb
TElektrons elektrons; //vektor elektronu
TMicros micros;  //vektor micro

Elektron * pickedElectron; //ukazatel na vybrany

int numElektrons=0;



extern bool stopFlag; //sepne se pri zmacknuti tlacitka stop
extern bool poprve; //pomocna promena
extern bool bReset; //ma se resetovat?
extern SoTimerSensor * tSensor; //na blikani elektrod
extern float velSim; //zpomaleni zrychleni simulace
extern bool bRedraw; //novy linac?

//forward
void resetElektrons(SoSeparator * root);

//////////////////////////////////////////////////////////////////
void sensorCallback(void * root, SoSensor *sensor) {
	SbTime dt, ct = SbTime::getTimeOfDay();

	if (lastTimeTick == SbTime::zero()) { //uplne poprve?
		lastTimeTick = ct; //ulozime current time
		sensor->schedule(); //naplanujeme dalsi zpusteni
		return; //vratime se
	}


	dt = ct - lastTimeTick;	//casovy usek od minuleho do ted

	//pohyb castic
	Elektron::updatePositions(&elektrons, 0, dt, lastTimeTick,(SoSeparator *) root);
	Micro::updatePositionsMicro(&micros, dt, lastTimeTick,(SoSeparator *) root);


	//ulozime pro dalsi jako lasttime
	lastTimeTick = ct;
	sensor->schedule(); //napslanujeme

	//-----------kontorly udalosti

	if (bRedraw) { //novy LINAC?
		deleteAllElektrons(&elektrons,(SoSeparator *) root);
		deleteAllMicros(&micros,(SoSeparator *) root);
		
		redrawLINAC();
		bRedraw=false;
		poprve=true;
	}

	if (bReset) { //nova simulace?
		resetElektrons((SoSeparator *) root);
		bReset=false;
	}




}

///init elektronu..jeden se zrovna vytvori
/////////////////////////////////////////////////////////////////
void createElektrons(SoSeparator *sceneRoot) {

    if (pickedElectron == NULL) {
		pickedElectron = new Elektron(0.0f, 0.0f, PozStartEl, 5.f, SbColor(0.9f, 0.0f, 0.0f),
		                             NULL,0,  0.0f, 0.0f, pocRychlost, sceneRoot);
	elektrons.push_back( pickedElectron );
	}else{
		elektrons.push_back(new Elektron(0.0f, 0.0f, PozStartEl, 5.f, SbColor(0.9f, 0.9f, 0.0f),
		                             NULL,0,  0.0f, 0.0f, pocRychlost, sceneRoot));
	}


	sensor = new SoOneShotSensor(sensorCallback, sceneRoot); //sensor pro pohyb
	sensor->schedule(); //naplanovani
}


////////////////////////////////////////////////////////////////
void create1Elektron(SoSeparator *sceneRoot) {
	
    if (pickedElectron == NULL) {
		pickedElectron = new Elektron(0.0f, 0.0f, PozStartEl, 5.f, SbColor(0.9f, 0.0f, 0.0f),
		                             NULL,0,  0.0f, 0.0f, pocRychlost, sceneRoot);
		elektrons.push_back( pickedElectron );
	} else {
		elektrons.push_back(new Elektron(0.0f, 0.0f, PozStartEl, 5.f, SbColor(0.9f, 0.9f, 0.0f),
		                             NULL,0,  0.0f, 0.0f, pocRychlost, sceneRoot));
	}

	

}

//reset simulace
///////////////////////////////////////////////////////////
void resetElektrons(SoSeparator * root){

  deleteAllElektrons(&elektrons,root);  
  deleteAllMicros(&micros,root); 
  poprve=true;
  pickedElectron = NULL;
  if (sensor !=NULL && !sensor->isScheduled()) sensor->unschedule();
  if (tSensor !=NULL && !tSensor->isScheduled()) tSensor->unschedule();
  create1Elektron(root); 
  sensor->schedule();

}

///animace burstu
/////////////////////////////////////////////////////
void createBurst(SoSeparator * sceneRoot, float  posZ, float el_velocity) {

	//vytvoreni x pozitronu o ruznych vektorech pohybu
	for (int i = 0; i<20;i++) {

		int delka_drahy=(float) (rand()%100 + 5);
		int smer_x=(rand()%2);
		int smer_y=(rand()%2);
		int Rvel_x=rand()% (int) (el_velocity/4)*20;
		int Rvel_y=rand()% (int) (el_velocity/4)*20;
		int vel_x=smer_x?(float) ( Rvel_x+ (el_velocity/100)):(-1)*((float) (Rvel_x+ (el_velocity/100)) );
		int vel_y=smer_y?(float) (Rvel_y + (el_velocity/100)):(-1)*((float) (Rvel_y + (el_velocity/100)) );
		int vel_z=(float) (el_velocity - (10*rand()) );

		//srand ( time(NULL) );	
	    micros.push_back( new Micro(0.0f, 0.0f, posZ, 0.9f, SbColor(0.9f, 0.0f, 0.0f),
		                             NULL, 
									 (float) delka_drahy ,
									 (float) vel_x,
									 (float) vel_y,
									 (float) vel_z,
									  sceneRoot)

	                    );


	} //for

}

///nalezne elektron ktery byl vybran
/////////////////////
void findElektron(SoSphere * sph) {

    //stary dame zpet
	if (pickedElectron!=NULL)
		 pickedElectron->material->diffuseColor.setValue(SbColor(0.9f, 0.9f, 0.0f));
	//najdem novy...nemusi existovat
		
		for (list<Elektron*>::iterator ci=elektrons.begin(); ci != elektrons.end(); ++ci) {
			
			if ((*ci)->sphere == sph) {
				
				(*ci)->material->diffuseColor.setValue(SbColor(0.9f, 0.0f, 0.0f));
				pickedElectron=(*ci); //ulozime vybrany
			}
		}	
}